pub mod prelude;

mod admin;
mod run;

use crate::{observe, prelude::*};
use prelude::*;

/// Web application for managing lyrics and live displays
#[derive(Parser)]
#[command(version, about, long_about = None)]
#[command(propagate_version = true)]
pub struct App {
    #[command(subcommand)]
    command: Commands,

    #[arg(global = true, long, value_enum, default_value = "info,lyrs=trace")]
    log_env_filter: String,
}

#[derive(Subcommand)]
enum Commands {
    /// Run the web application server
    Run(run::Run),

    /// Perform administrator actions
    Admin(admin::Admin),
}

#[derive(Error, Debug)]
pub enum RunError {
    #[error("run error: {0}")]
    Run(#[from] run::RunError),

    #[error("admin error: {0}")]
    Admin(#[from] admin::RunError),

    #[error("{0}")]
    Eyre(#[from] color_eyre::Report),
}

pub async fn run() -> Result<(), RunError> {
    let cli = App::parse();
    observe::setup_logging(&cli.log_env_filter)?;
    match cli.command {
        Commands::Run(args) => Ok(args.run().await?),
        Commands::Admin(args) => Ok(args.run().await?),
    }
}
