use super::prelude::*;

#[derive(Args)]
pub struct Admin {
    /// Register a user account
    #[command(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    CreateAccount(CreateAccount),
}

#[derive(Error, Debug)]
pub enum RunError {
    #[error("{0}")]
    Eyre(#[from] color_eyre::Report),

    #[error("create account error: {0}")]
    CreateAccount(#[from] CreateAccountError),
}

impl Admin {
    pub async fn run(&self) -> Result<(), RunError> {
        match &self.command {
            Commands::CreateAccount(args) => Ok(args.run().await?),
        }
    }
}

/// Create a user account
#[derive(Args)]
pub struct CreateAccount {
    /// Whether or not the user is a site super admin
    #[arg(short, long, default_value = "false")]
    pub superadmin: bool,

    /// The email address of the user account
    #[arg(short = 'e', long)]
    pub email_address: String,

    /// The user's initial password - if none is set, a random one will be used and output
    #[arg(short, long)]
    pub initial_password: Option<String>,
}

#[derive(Error, Debug)]
pub enum CreateAccountError {}

impl CreateAccount {
    pub async fn run(&self) -> Result<(), CreateAccountError> {
        // self.email_address
        let password = self.initial_password.unwrap_or_else(|| {
            rand::thread_rng()
                .sample_iter(&Alphanumeric)
                .take(7)
                .map(char::from)
                .collect()
        });

        Ok(())
    }
}
