use crate::{
    file_watcher::{file_monitor, prelude::*},
    prelude::*,
};

use axum::Router;
use std::path::PathBuf;
use std::str::FromStr;
use std::sync::OnceLock;
use tower_http::services::ServeDir;
use tower_livereload::Reloader;

fn static_file_dir() -> &'static PathBuf {
    static STATIC_FILE_DIR: OnceLock<PathBuf> = OnceLock::new();
    STATIC_FILE_DIR.get_or_init(|| PathBuf::from_str("static").unwrap())
}

pub fn router(reloader: Option<Reloader>) -> Result<(Router, Option<FileWatcher>), notify::Error> {
    let watcher = if let Some(rl) = reloader {
        // TODO: debounce?
        Some(file_monitor(static_file_dir(), move || {
            info!("Static File Watcher Event");
            rl.reload()
        })?)
    } else {
        None
    };

    let router = Router::new().nest_service("/", ServeDir::new(static_file_dir()));
    Ok((router, watcher))
}
